import Vue from 'vue';
import { forEach, isArray, merge } from 'lodash';
import { serialize, parse } from 'cookie';

const nuxtServerInit = async ({ commit }, { app, req, res, query, env }) => {
  const cookies = parse(req.headers.cookie || '');

  let apiToken = cookies.apiToken || env.API_TOKEN;

  if (query.apiToken) {
    apiToken = query.apiToken;
    res.setHeader('Set-Cookie', serialize('apiToken', apiToken, { maxAge: 3600 }));
  }

  commit('API_TOKEN', apiToken);

  if (app.$init) {
    await app.$init();
  }
};

// Custom serializer for Lambda
const paramsSerializer = (params) => {
  const paramsArray = [];

  forEach(params, (value, key) => {
    if (isArray(value)) {
      value.forEach((v, i) => {
        paramsArray.push(`${key}[${i}]=${v}`);
      });
    } else {
      paramsArray.push(`${key}=${value}`);
    }
  });

  return encodeURI(paramsArray.join('&'));
};

const setPayload = (state, map, payload) => {
  state[map] = { ...state[map], ...payload };
};

export const state = () => ({
  apiToken: undefined,
  role: 'guest',
  config: {},
  metadata: {
    title: '',
    description: '',
  },
  userAgent: {},
  device: {},
  misc: {},
  history: [],
  taxonomies: {},
  entities: {},
});

export const mutations = {
  API_TOKEN(state, apiToken) {
    Vue.set(state, 'apiToken', apiToken);
  },
  ROLE(state, role) {
    Vue.set(state, 'role', role);
  },
  CONFIG(state, config) {
    Vue.set(state, 'config', config);
  },
  METADATA(state, payload) {
    setPayload(state, 'metadata', payload);
  },
  USERAGENT(state, payload) {
    setPayload(state, 'userAgent', payload);
  },
  DEVICE(state, payload) {
    setPayload(state, 'device', payload);
  },
  MISC(state, payload) {
    setPayload(state, 'misc', payload);
  },
  HISTORY(state, fromState) {
    state.history.unshift(fromState);
    Vue.set(state, 'history', state.history);
  },
  TAXONOMY(state, taxonomy) {
    Vue.set(state.taxonomies, taxonomy.slug, taxonomy);
  },
  ENTITY(state, entity) {
    if (!entity._id) {
      return;
    }
    Vue.set(state.entities, entity._id, merge(state.entities[entity._id] || {}, entity));
  },
  ENTITIES(state, entities) {
    entities = entities.rows || entities.docs || entities;
    entities.forEach((entity) => {
      let id;
      if (entity._id) {
        id = entity._id;
      } else {
        id = entity.id;
        entity = entity.doc || entity.fields || entity;
        id = entity._id || id;
      }
      if (!id) {
        return;
      }
      Vue.set(state.entities, id, merge(state.entities[id] || {}, entity));
    });
  },
};

export const actions = {
  nuxtServerInit,
  async fetchConfig({ commit }) {
    const result = await this.$axios.get('config');
    commit('ROLE', result.headers['x-role']);
    commit('CONFIG', result.data);
    return result;
  },
  async fetchMetadata({ commit }) {
    const result = await this.$axios.get('metadata');
    commit('ROLE', result.headers['x-role']);
    commit('METADATA', result.data);
    return result;
  },
  async fetchTaxonomy({ commit }, { slug }) {
    const result = await this.$axios.$get('taxonomy', { params: { slug } });
    commit('TAXONOMY', result);
    return result;
  },
  async fetchEntities({ commit }, { method, path, params, data }) {
    const result = await this.$axios.$request({
      url: path || 'entities',
      method: method || 'get',
      params,
      paramsSerializer,
      data,
    });
    commit('ENTITIES', result);
    return result;
  },
};
